#!/usr/bin/env python3

"""
NAME:
	find_aliases.py - find all the names that map to a given address.

SYNOPSIS:
	find_aliases.py [options] "ip" ...

DESCRIPTION:
	This tool finds all the names by which "ip" is known.
	If "ip" is a domain name, it is first resolved to an IP
	address.

	Output lists the IP address, followed by the canonical name
	(from the PTR record) and then all the other names.
	
SEE ALSO:
	getdata.py(1)

AUTHOR:
	Simon J. Gerraty <sjg@crufty.net>

"""

"""
RCSid:
	$Id: find_aliases.py,v 1.2 2023/06/06 22:48:57 sjg Exp $

	@(#) Copyright (c) 1997-2023 Simon J. Gerraty.

	This file is provided in the hope that it will
	be of use.  There is absolutely NO WARRANTY.
	Permission to copy, redistribute or otherwise
	use this file is hereby granted provided that 
	the above copyright notice and this notice are
	left intact. 
      
	Please send copies of changes and bug-fixes to:
	sjg@crufty.net
"""

import sys
import ipaddress
import iputils
import dnsutils
import conf as cf

As = {}
CNAMEs = {}
Names = {}
PTRs = {}

def load_data(conf):

    global As
    global CNAMEs
    global Names
    global PTRs

    inf = conf.get('data', sys.stdin)
    for line in inf.readlines():
        w = line.strip().split()
        if not w[0] in ['A', 'AAAA', 'CNAME', 'PTR']:
            continue
        if w[0] == 'PTR':
            if 'ip6' in w[1]:
                ip = iputils.arpa2ip(w[1])
            else:
                # check for CIDR
                a = w[1].strip('.in-addr.arpa').split('.')
                if len(a) > 4:
                    n = []
                    for x in a:
                        if x.isdigit():
                            n.append(x)
                    a = n
                a.reverse()
                ip = '.'.join(a)
            Names[ip] = [w[2]]
            PTRs[ip] = w[2]
            As[w[2]] = ip
            continue
        if w[0] in ['A', 'AAAA']:
            As[w[1]] = w[2]
            if not w[2] in Names:
                Names[w[2]] = []
            Names[w[2]].append(w[1])
            continue
        if w[0] == 'CNAME':
            Names[w[2]].append(w[1])
            CNAMEs[w[1]] = w[2]

def find_ip(name):
    global CNAMEs
    global As
    if name in CNAMEs:
        name = CNAMEs[name]
    if name in As:
        return As[name]
    if not '.' in name:
        name += '.'
        for n in As.keys():
            if n.startswith(name):
                return As[n]
    return None

def find_names(ip):
    global Names
    global PTRs

    names = []
    for n in Names.get(ip):
        if n in names:
            continue
        names.append(n)
        if n in Names:
            nl = find_names(n)
            names += nl
    return names

def find_aliases(args, conf):
    global Names
    global PTRs
    
    load_data(conf)

    for a in args:
        try:
            ip = str(ipaddress.ip_address(a))
        except:
            ip = find_ip(a)
            pass
        names = find_names(ip)
        ptr = PTRs.get(ip)
        if ptr in names:
            names.remove(ptr)

        print('{}\t{}\t{}'.format(ip,ptr,' '.join(names)))

def myopts(o, a, conf):
    if o == '-D':
        conf['data'] = open(a, 'r')

if __name__ == '__main__':

    conf,args = dnsutils.app_init('D:', myopts, None)

    find_aliases(args, conf)
    sys.exit(0)

