# NAME:
#	dnsmagic.mk - make agnostic rules for DNSMagic
#
# DESCRIPTION:
#
#	This makefile is intended to be make agnostic, so we avoid
#	syntax which is not supported by both BSD make and GNU make.
#	It is intended to be included by Makefile or GNUmakefile.
#
#	If your make(1) does not support 'include'ing files, then
#	remove the -N option from 'dnsdeps' and it will append
#	.depend to the makefile.
#	You would do better to use one of the above.
#
#	The setup here relies on SOA records being kept in files named
#	zone.soa which include zone.db where the data is kept.
#	The zone.db files are tracked in an SCM, zone.soa files are
#	generated/updated by make using ${UPDSOA}.
#
#	'make depend' will ensure that all the primary zone files
#	referenced by named.conf are recursively checked 
#	for dependencies.
#
#	'make' will update the serial number of any .soa files which
#	are out of date.
#
#	'make regress' will run our regression suite.
#	We do this before any commit to the SCM.
#
# AUTHOR:
#	Simon J. Gerraty <sjg@crufty.net>
#

# RCSid:
#	$Id: dnsmagic.mk,v 1.12 2023/06/11 18:36:05 sjg Exp $
#
#	@(#) Copyright (c) 1995-2023 Simon J. Gerraty.
#
#	This file is provided in the hope that it will
#	be of use.  There is absolutely NO WARRANTY.
#	Permission to copy, redistribute or otherwise
#	use this file is hereby granted provided that 
#	the above copyright notice and this notice are
#	left intact. 
#      
#	Please send copies of changes and bug-fixes to:
#	sjg@crufty.net
#

# since we often run from cron(8), ensure we have
# what we want in the PATH...
LOCAL ?= /usr/local
SHARE ?= ${LOCAL}/share
DNSMAGIC ?= ${SHARE}/dns

# we have .sh .pl and .py variants
# the basename is a symlink the preferred one
# eg updsoa -> updsoa.py
DNSDEPS ?= dnsdeps
SECONDARY_ZONES ?= secondary-zones
UPDREV ?= updrev
UPDSOA ?= updsoa

# lose the -N if you need the dependencies appended to makefile.
# better yet, get bmake
DNSDEPS_FLAGS = -N
MKDEP = ${DNSDEPS} ${DNSDEPS_FLAGS}
BOUNCE = bouncedns
RELOAD = bouncedns reload
DNS_REGRESS = ${DNSMAGIC}/regress -v
UPDREV_FLAGS = -N ${PZONES} ${UPDREV_OPTS} ${NDDUMP}
UPDREV_DEPS = ${NDDUMP}
# An SCM agnostic update mechanism
SCM_UPDATE ?= scm-update.sh

# bind9 is a pain to work with
# with earlier versions we could use 0
# but at least rndc now waits for dump to complete
ND_PORT ?= 53953
NDC_PORT ?= 5953
CONF_TEST_SED = -e 's/port 953/port ${NDC_PORT}/'

# if we are acting as secondary for other sites set
# BZONES = backup.zones in makefile
PZONES = primary.zones
SZONES = secondary.zones

NAMED_CONF ?= named.conf
SECONDARY_CONF = ${NAMED_CONF}.sec secondary.zones
NDDUMP = named_dump.db

# probably best not to do this here, as make(1)
# has already built a dependency graph, so if this
# does update anything, make needs to be re-run anyway.
#_SCM = .scm

all:	${_SCM} .depend zones ${SECONDARY_CONF} ${NAMED_CONF}.test

# this avoids the need to [.]export PATH
#DNSMAGIC_ENV += PATH=./bin:${DNSMAGIC}/bin:${SHARE}/bin:$$PATH:${LOCAL}/bin:/etc/rc_d
# but we can do that from the parent makefile

# this lets us invoke updsoa and bounce once only
.BEGIN: prezones
prezones:
	@rm -f .zones
	@[ -d secondary ] || mkdir secondary

zones:	.zones
	@test -s .zones && ${DNSMAGIC_ENV} xargs ${UPDSOA} ${UPDSOA_FLAGS} < .zones || :
	@test -s .zones && { echo ${RELOAD}; DNSDIR=. ${DNSMAGIC_ENV} ${RELOAD} > /dev/null; } || :

# dependencies for .zones are in .depend
# the .zones below is just to keep make quiet
# when .depend does not yet exist
.zones:

.PRECIOUS:	.depend
.depend depend:	${NAMED_CONF} ${PZONES} ${PZONE_DEPS}
	@echo ${MKDEP} ${NAMED_CONF}
	@${DNSMAGIC_ENV} ${MKDEP} ${NAMED_CONF}

# we don't use the usual rules
.SUFFIXES:

# edit this to match what you use - but you'll have to fix bin/* too.
.SUFFIXES: .db .rev .soa .conf .conf.sec .conf.test

.db.soa .rev.soa:
	@echo '${UPDSOA} $@'; echo $@ >> .zones

# for secondary servers
.conf.conf.sec:
	sed -e 's/primary.zones/secondary.zones/' ${CONF_SEC_SED} < $? > $@
	@echo ${BOUNCE}; DNSDIR=. ${DNSMAGIC_ENV} ${BOUNCE} > /dev/null

secondary.zones:	primary.zones
	@echo '${SECONDARY_ZONES} ${DNS_PRIMARIES} < $? > $@ || rm -f $@'
	@${DNSMAGIC_ENV} ${SECONDARY_ZONES} ${DNS_PRIMARIES} < $? > $@ || rm -f $@
	@echo ${BOUNCE}; DNSDIR=. ${DNSMAGIC_ENV} ${BOUNCE} > /dev/null

# we need to reload when secondary.zones changes too.
.seconds:	secondary.zones ${BZONES}
	@echo ${BOUNCE}; DNSDIR=. ${DNSMAGIC_ENV} ${RELOAD} > /dev/null
	@touch $@

.conf.conf.test:
	sed ${CONF_TEST_SED} -e '/query-source/d' \
	-e '/listen-on/d' -e '/pid-file/d' -e '/session-keyfile/d' \
	-e 's,^[ 	]*directory.*,listen-on port ${ND_PORT} { 127.0.0.1; }; pid-file "./named.pid"; session-keyfile "./session.key";,' \
	-e '/rndc.key/s,/.*/,./,' \
	-e '/secondary/d' $? > $@

regress .regress.ests:	.depend ${NAMED_CONF}.test
	${DNSMAGIC_ENV} ${DNS_REGRESS}

# this is not an automagic target as not everyone wants it.
revs:	.regress.ests ${UPDREV_DEPS}
	@echo ${UPDREV} ${UPDREV_FLAGS}
	@${DNSMAGIC_ENV} ${UPDREV} ${UPDREV_FLAGS}

.scm:
	@echo ${SCM_UPDATE}; test -f .noscm || ${SCM_UPDATE}

# for git and hg we need to do some config
# the first time we are run
all: .dnsmagicrc
.dnsmagicrc:
	@{ echo "DNSMAGIC=${DNSMAGIC}; export DNSMAGIC"; \
	test -d ${DNSMAGIC}/etc/rc_d && echo "RC_D=${DNSMAGIC}/etc/rc_d; export RC_D"; } > $@
	@test ! -d .git || ln -sf ${DNSMAGIC}/pre-commit.sh .git/hooks/pre-commit
	@test ! -d .hg || { echo '[hooks]'; echo precommit = ${DNSMAGIC}/pre-commit.sh; } >> .hg/hgrc
